const canvas = document.getElementById('snakeCanvas');
const ctx = canvas.getContext('2d');

const gridSize = 15;
const scale = 200;
canvas.width = gridSize * scale;
canvas.height = gridSize * scale;

let snake = [{ x: 10, y: 10 }];
let direction = { x: 0, y: 0 };
let food = { x: 5, y: 5 };
let score = 0;
let gameOver = false;

let token = null;

function draw() {
    ctx.clearRect(0, 0, canvas.width, canvas.height);

    // draw grid of checkerboard pattern
    for (let x = 0; x < gridSize; x++) {
        for (let y = 0; y < gridSize; y++) {
            ctx.fillStyle = (x + y) % 2 === 0 ? 'darkgray' : 'grey';
            ctx.fillRect(x * scale, y * scale, scale, scale);
        }
    }

    // Draw snake
    snake.forEach(segment => {
        ctx.fillStyle = (snake.indexOf(segment) % 2 === 0) ? 'green' : 'darkgreen';
        ctx.fillRect(segment.x * scale, segment.y * scale, scale, scale);
    });

    // Draw food
    ctx.fillStyle = 'red';
    ctx.fillRect(food.x * scale, food.y * scale, scale, scale);
}

function update() {
    if (gameOver) return;

    // Move snake
    const head = { x: snake[0].x + direction.x, y: snake[0].y + direction.y };

    // Add new head
    snake.unshift(head);

    // Check for food collision
    if (head.x === food.x && head.y === food.y) {
        score += 10; // Increase score
        placeFood();
    } else {
        snake.pop(); // Remove tail if no food eaten
    }

    // Check for wall collision
    if (head.x < 0 || head.x >= gridSize || head.y < 0 || head.y >= gridSize) {
        gameOver = true;
        alert(`Game Over! Your score: ${score}`);
        return;
    }

    // Check for self collision
    for (let i = 1; i < snake.length; i++) {
        if (head.x === snake[i].x && head.y === snake[i].y) {
            gameOver = true;
            alert(`Game Over! Your score: ${score}`);
            return;
        }
    }
}

function placeFood() {
    do {
        food.x = Math.floor(Math.random() * gridSize);
        food.y = Math.floor(Math.random() * gridSize);
    } while (snake.some(segment => segment.x === food.x && segment.y === food.y));
}

function changeDirection(event) {
    switch (event.key) {
        case 'w':
            if (direction.y === 0) direction = { x: 0, y: -1 };
            break;
        case 's':
            if (direction.y === 0) direction = { x: 0, y: 1 };
            break;
        case 'a':
            if (direction.x === 0) direction = { x: -1, y: 0 };
            break;
        case 'd':
            if (direction.x === 0) direction = { x: 1, y: 0 };
            break;
    }
}

function gameLoop() {
    if (!gameOver) {
        update();
        draw();
        setTimeout(gameLoop, 100);
    }
}

document.addEventListener('keydown', changeDirection);
// Start the game loop
gameLoop();
// Initial draw
draw();


const widget = document.querySelector("#cap");

widget.addEventListener("solve", function (e) {
    token = e.detail.token;
    console.log("Captcha solved, token:", token);
});

// Function to submit score to the server
document.getElementById("snakeForm").addEventListener('submit', function(event) {
    event.preventDefault();

    if (!token) {
        alert('Please complete the CAPTCHA before submitting your score.');
        return;
    }

    // post request to server with score
    fetch('/snake/submit', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify({
            score: score,
            name: document.getElementById('name').value,
            cap: token
        })
    }).then(response => {
        if (response.ok) {
            alert('Score submitted successfully!');
            window.location.reload();
        } else {
            alert('Failed to submit score, check console for details.');
            response.json().then(data => console.error(data));
        }
    }).catch(error => {
        console.error('Error submitting score:', error);
        alert('Error submitting score, check console for details.');
    });
});